   // Converts the Set Splitting Problem to HCP. It is assumed the instance is in simple form, that is:
   //
   // 1) No set contains only a single entry (or else the instance is trivially unsolvable)
   // 2) Each element from the Universe set is used in at least one set (or else it can be placed in either partition so is inconsequential)
   // 3) No set is equal to the Universe set (otherwise it has no bearing on the solution)
   // 4) No set is a proper subset of another set (or else the larger set can be discarded)
   //
   // The input should be in the following form. The first line gives the cardinality of the Universe set, the second line gives the number
   // of subsets. Then each subsequent line corresponds to a subset, with each present entry listed. An example is the following:
   //
   // 6
   // 4
   // 1 2 5 6
   // 2 3 4
   // 1 3 6
   // 4 5
   //
   // A solution of the above example is U1 = [1 2 3 5] and U2 = [4 6]
   //
   //
   // Written by Michael Haythorpe
    

   import java.io.*;
   import java.util.StringTokenizer;
   import java.util.Arrays;

   public class SSP2HCP
   {
      public static void main(String [] args)
      {
         if(args.length < 1 || args.length > 2)
         {
            System.out.println("SSP2HCP should be run using the following input:");
            System.out.println();
            System.out.println("SSP2HCP.exe <input_file.ssp> <output_file.hcp>");
            System.out.println();
            System.out.println("If no output filename is specified, SSP2HCP creates input_file.hcp.");
            return;
         }
      
         String inputFilename = "";
         String outputFilename = "";
         int literals = 0;
         int clauses = 0;
      
         if(args.length == 1)
         {
            inputFilename = args[0];
            outputFilename = inputFilename;
            if(outputFilename.substring(outputFilename.length()-4).equals(".ssp"))
               outputFilename = outputFilename.substring(0,outputFilename.length()-4);
            outputFilename += ".hcp";
         }
      
         if(args.length == 2)
         {
            inputFilename = args[0];
            outputFilename = args[1];
         }
      
         File inputFile = new File(inputFilename);
         File outputFile = new File(outputFilename);
      			
         BufferedReader br = null;
         int u = 0;
         int []sizes = null;
         int s = 0;
         int [][]subsets = null;
         int c = 0;
      	
         try
         {
            br = new BufferedReader(new FileReader(inputFile));
         }
            catch(Exception e)
            {
               System.out.println("ERROR: Cannot read file " + inputFilename);
               return;
            }		
      	
         String line = "";
         try
         {
            line = br.readLine();
            u = Integer.parseInt(line);
            line = br.readLine();
            s = Integer.parseInt(line);
            sizes = new int[s];
            subsets = new int[s][u];
         }
         
            catch(Exception e)
            {
               System.out.println("ERROR: First two lines should contain two integers.");
               return;
            }
      	
         for(int i=0; i<s; i++)
         {
            try
            {
               line = br.readLine();
            }
               catch(Exception e)
               {
                  System.out.println("ERROR: " + inputFilename + " specified " + s + " subsets, but only " + i + " are contained in the file!");
                  return;
               }
            
            try
            {
               StringTokenizer tokens = new StringTokenizer(line);
               int count = 0;
               while(tokens.hasMoreTokens())
                  subsets[i][count++] = Integer.parseInt(tokens.nextToken());
            
               sizes[i] = count;
               c += count;
            }
               catch(Exception e)
               {
                  System.out.println("ERROR: " + inputFilename + " should only contain integers! File contains: " + line);
                  return;
               }
         }
         try
         {
            br.close();
         }
            catch(Exception e)
            {
               System.out.println(e);
            }
      	
      	// Now to produce the graph.
      
         int N = 4*u + 5*s + 6*c + 2;
         int arcs[][] = new int[N][Math.max(4,u+1)];
         int []degrees = new int[N];
      
      // For each element of U
      
         for(int i=0; i<u; i++)
         {
            arcs[4*i][degrees[4*i]++] = 4*i+1;
            arcs[4*i][degrees[4*i]++] = 4*i+2;
            arcs[4*i+1][degrees[4*i+1]++] = 4*i;
            arcs[4*i+1][degrees[4*i+1]++] = 4*i+2;
            arcs[4*i+2][degrees[4*i+2]++] = 4*i;
            arcs[4*i+2][degrees[4*i+2]++] = 4*i+1;
            arcs[4*i+3][degrees[4*i+3]++] = 4*i+4;
         }
         
      // For each subset Si
      
         int []sc = new int[s+1];
         sc[0] = 4*u;
         for(int i=1; i<s+1; i++)
            sc[i] = sc[i-1] + 5 + 6*sizes[i-1];
      
         for(int i=0; i<s; i++)
         {
            arcs[sc[i]+1][degrees[sc[i]+1]++] = sc[i]+6;
            arcs[sc[i]+1][degrees[sc[i]+1]++] = sc[i]+6+3*sizes[i];
            arcs[sc[i]+3][degrees[sc[i]+3]++] = sc[i]+4;
            arcs[sc[i]+4][degrees[sc[i]+4]++] = sc[i]+3;
            arcs[sc[i]+4][degrees[sc[i]+4]++] = sc[i]+5;
            arcs[sc[i]+5][degrees[sc[i]+5]++] = sc[i]+4;
            arcs[sc[i]+5+3*sizes[i]][degrees[sc[i]+5+3*sizes[i]]++] = sc[i]+2;
            arcs[sc[i]+5+6*sizes[i]][degrees[sc[i]+5+6*sizes[i]]++] = sc[i]+2;
                     	
            for(int j=0; j<sizes[i]; j++)
            {
               arcs[sc[i]+6+3*j][degrees[sc[i]+6+3*j]++] = sc[i]+7+3*j;
               arcs[sc[i]+7+3*j][degrees[sc[i]+7+3*j]++] = sc[i]+6+3*j;
               arcs[sc[i]+7+3*j][degrees[sc[i]+7+3*j]++] = sc[i]+8+3*j;
               arcs[sc[i]+8+3*j][degrees[sc[i]+8+3*j]++] = sc[i]+7+3*j;
               arcs[sc[i]+6+3*j+3*sizes[i]][degrees[sc[i]+6+3*j+3*sizes[i]]++] = sc[i]+7+3*j+3*sizes[i];
               arcs[sc[i]+7+3*j+3*sizes[i]][degrees[sc[i]+7+3*j+3*sizes[i]]++] = sc[i]+6+3*j+3*sizes[i];
               arcs[sc[i]+7+3*j+3*sizes[i]][degrees[sc[i]+7+3*j+3*sizes[i]]++] = sc[i]+8+3*j+3*sizes[i];
               arcs[sc[i]+8+3*j+3*sizes[i]][degrees[sc[i]+8+3*j+3*sizes[i]]++] = sc[i]+7+3*j+3*sizes[i];
            }
            
            for(int j=0; j<sizes[i]-1; j++)
            {
               arcs[sc[i]+8+3*j][degrees[sc[i]+8+3*j]++] = sc[i]+9+3*j;
               arcs[sc[i]+8+3*j+3*sizes[i]][degrees[sc[i]+8+3*j+3*sizes[i]]++] = sc[i]+9+3*j+3*sizes[i];
               arcs[sc[i]+8+3*j][degrees[sc[i]+8+3*j]++] = sc[i]+9+3*j+3*sizes[i];
               arcs[sc[i]+8+3*j+3*sizes[i]][degrees[sc[i]+8+3*j+3*sizes[i]]++] = sc[i]+9+3*j;
               arcs[sc[i]+8+3*j][degrees[sc[i]+8+3*j]++] = sc[i]+3;
               arcs[sc[i]+8+3*j+3*sizes[i]][degrees[sc[i]+8+3*j+3*sizes[i]]++] = sc[i]+5;
               arcs[sc[i]+3][degrees[sc[i]+3]++] = sc[i]+9+3*j;
               arcs[sc[i]+5][degrees[sc[i]+5]++] = sc[i]+9+3*j+3*sizes[i];
            }
            
         	arcs[sc[i]+2][degrees[sc[i]+2]++] = sc[i+1]+1;
         }
      	
      	// Now for the two V^C vertices
      	
         arcs[4*u][degrees[4*u]++] = 4*u+1;
         arcs[N-1][degrees[N-1]++] = 0;
      	
      	// Finally, to put in the edges for each subset
      	
         for(int i=0; i<u; i++)
         {
            int last1 = 4*i+1;
            int last2 = 4*i+2;
            for(int j=0; j<s; j++)
            {
               for(int k=0; k<sizes[j]; k++)
               {
                  if(subsets[j][k] == i+1)
                  {
                     arcs[last1][degrees[last1]++] = sc[j]+8+3*sizes[j]+3*k;
                     arcs[last2][degrees[last2]++] = sc[j]+8+3*k;
                     last1 = sc[j]+6+3*sizes[j]+3*k;
                     last2 = sc[j]+6+3*k;
                  }
               }	
            }	
            arcs[last1][degrees[last1]++] = 4*i+3;
            arcs[last2][degrees[last2]++] = 4*i+3;
         }
         
      	// Now to turn the graph into an undirected graph
      	
         int []undDegrees = new int[3*N];
         int [][]undEdges = new int[3*N][Math.max(5,u+2)];
      	
         for(int i=0; i<N; i++)
         {
            undEdges[3*i][undDegrees[3*i]++] = 3*i+1;
            undEdges[3*i+1][undDegrees[3*i+1]++] = 3*i;
            undEdges[3*i+1][undDegrees[3*i+1]++] = 3*i+2;
            undEdges[3*i+2][undDegrees[3*i+2]++] = 3*i+1;
         }
      	
         for(int i=0; i<N; i++)
         {
            for(int j=0; j<degrees[i]; j++)
            {
               undEdges[3*i+2][undDegrees[3*i+2]++] = 3*arcs[i][j];
               undEdges[3*arcs[i][j]][undDegrees[3*arcs[i][j]]++] = 3*i+2;
            }	
         }
      	
      	// Finally, to write the graph to a file
      
         try
         {
            BufferedWriter bw = new BufferedWriter(new FileWriter(outputFile));
         
            bw.write("NAME : Adapted from SSP file " + inputFilename);
            bw.newLine();
            bw.write("COMMENT : ");
            bw.newLine();
            bw.write("TYPE : HCP");
            bw.newLine();
            bw.write("DIMENSION : " + 3*N);
            bw.newLine();
            bw.write("EDGE_DATA_FORMAT : EDGE_LIST");
            bw.newLine();
            bw.write("EDGE_DATA_SECTION");
            bw.newLine();
         
            for(int i=0; i<3*N; i++)
            {
               int array[] = new int[undDegrees[i]];
               for(int j=0; j<undDegrees[i]; j++)
                  array[j] = undEdges[i][j]+1;
                  
               Arrays.sort(array);
               for(int j=0; j<undDegrees[i]; j++)
               {
                  bw.write((i+1) + " " + array[j]);
                  bw.newLine();
               }
            }
         
            bw.write("-1");
            bw.newLine();
            bw.write("EOF");
            bw.newLine();
         
            bw.close();
         }
            catch(Exception e)
            {
               System.out.println(e);
            }
            
         System.out.println("Final graph contains " + (3*N) + " vertices.");
      }   
   }